const CACHE_VERSION = "4.4.0.0";
self.addEventListener("install", function (event) {
	self.skipWaiting();
	event.waitUntil(
		caches.open(CACHE_VERSION).then(function (cache) {
			console.log(self.__WB_MANIFEST);
			return cache.addAll(self.__WB_MANIFEST || []);
		})
	);
});

self.addEventListener("message", (event) => {
	if (event.data === "skipWaiting") {
		self.skipWaiting();
	}
});

var whitelist = /at\.alicdn\.com/;
/* 注册fetch事件，拦截全站的请求 */
self.addEventListener("fetch", function (event) {
	var request = event.request;
	if (!request.url.startsWith(location.origin) && !whitelist.test(request.url))
		return console.log("ignore:", request.url);
	var pms;
	if (request.mode == "navigate") {
		pms = NetworkFirst(event).then(function (res) {
			return res;
		});
	}
	if (~["style", "script", "worker"].indexOf(request.destination)) {
		pms = StaleWhileRevalidate(event).then(function (res) {
			return res;
		});
	}
	if (~["image", "font"].indexOf(request.destination)) {
		pms = CacheFirst(event).then(function (res) {
			return res;
		});
	}
	if (pms) event.respondWith(pms);
});

function StaleWhileRevalidate(event) {
	return matchCache(event).then(function (res) {
		if (res) {
			requestBackend(event);
			return res;
		}
		return requestBackend(event);
	});
}

function CacheFirst(event, ttl) {
	ttl = ttl || 86400e3;
	return matchCache(event).then(function (res) {
		if (res && res.headers.get("x-sw-cache-timestamp") > Date.now() - ttl) return res;
		if (!res) return requestBackend(event);
		return requestBackend(event).then(function (resp) {
			return resp || res;
		});
	});
}

function NetworkFirst(event) {
	return requestBackend(event).then(function (res) {
		return res || matchCache(event);
	});
}

function matchCache(event) {
	return caches.match(event.request).then(function (res) {
		return res || event.preloadResponse;
	});
}

function requestBackend(event) {
	var request = event.request.clone();
	return fetch(request)
		.catch(function (err) {
			console.error(err);
		})
		.then(function (res) {
			if (!res || res.status != 200 || res.type != "basic") {
				return res;
			}
			cloneResponse(res, {"x-sw-cache-timestamp": Date.now()}).then(function (response) {
				caches.open(CACHE_VERSION).then(function (cache) {
					cache.put(event.request, response);
				});
			});

			return res;
		});
}

function cloneResponse(response, extraHeaders) {
	if (!response) {
		return Promise.resolve();
	}

	var res = response.clone();

	var init = {
		status: res.status,
		statusText: res.statusText,
		headers: extraHeaders || {},
	};

	res.headers.forEach(function (val, key) {
		init.headers[key] = val;
	});

	return res.blob().then(function (blob) {
		return new Response(blob, init);
	});
}
